#include "BeNESJoypad.h"
#include <interface/InterfaceDefs.h>
#include <device/Joystick.h>

#include <stdio.h>
#include <unistd.h>

const char *PORTNAME="gameport/201";

inline bool isKeyDown(key_info& info, unsigned char key)
{
	return (info.key_states[key>>3] & (1<<(7-(key&7))));
}

#define RETURN 0x47
#define SPACE 0x5e
#define UP 0x57
#define DOWN 0x62
#define LEFT 0x61
#define RIGHT 0x63
#define LSHIFT 0x4b
#define Z 0x4c
#define X 0x4d

#define PTOLERANCE 100
#define NTOLERANCE -100

BeNESJoypad::BeNESJoypad(): NesJoypad()
{
	mThread = spawn_thread(BeNESJoypad::Polling, 
		"BeNESJoy", B_NORMAL_PRIORITY, this);
	resume_thread(mThread);
	inputType=2;
	mRunning = true;
}

BeNESJoypad::~BeNESJoypad()
{
	if (mThread) {
		status_t result;
		mRunning = false;
		wait_for_thread(mThread, &result);
		kill_thread(mThread);
	}
}

void BeNESJoypad::setInput(int type)
{
	inputType = type;
}	

int32 BeNESJoypad::Polling(void* arg)
{
	const int pollingRate = 1000000 / 60;
	BJoystick bjoy;
	status_t st;
	int32 buttonstat;
	int16 axesstat[2];
	BeNESJoypad* joy = reinterpret_cast<BeNESJoypad*>(arg);
	key_info info;
	int type;
	
	st = bjoy.Open(PORTNAME);
	
	while (joy->mRunning)
	{
		snooze(pollingRate);
		type = joy->inputType;	

		if (type==2)
		{
			get_key_info(&info);
		
			if (isKeyDown(info, Z)) joy->pushA();
			else joy->releaseA();
		
			if (isKeyDown(info, X) || isKeyDown(info, LSHIFT)) joy->pushB();
			else joy->releaseB();
	
			if (isKeyDown(info, RETURN)) joy->pushStart();
			else joy->releaseStart();

			if (isKeyDown(info, SPACE)) joy->pushSelect();
			else joy->releaseSelect();

			if (isKeyDown(info, UP)) joy->pushUp();
			else joy->releaseUp();

			if (isKeyDown(info, DOWN)) joy->pushDown();
			else joy->releaseDown();

			if (isKeyDown(info, LEFT)) joy->pushLeft();
			else joy->releaseLeft();

			if (isKeyDown(info, RIGHT)) joy->pushRight();
			else joy->releaseRight();
		}
		else if(type==1)
		{
			joy->releaseA();
			joy->releaseB();
			joy->releaseSelect();
			joy->releaseStart();
			joy->releaseUp();
			joy->releaseDown();
			joy->releaseLeft();
			joy->releaseRight();
		
			bjoy.Update();
			buttonstat = bjoy.ButtonValues();
			bjoy.GetAxisValues(axesstat);
		
			if (buttonstat)
			{
				if ((buttonstat & 8)==8) joy->pushA();
			
				if((buttonstat & 16) == 16) joy->pushB();
			
				if((buttonstat & 256) == 256) joy->pushStart();
			
				if ((buttonstat & 512) == 512) joy->pushSelect();
			}
			if (*(axesstat+1))
			{
				if (*(axesstat+1) > 0)
				{
					joy->pushUp();
				}
				else if (*(axesstat+1) < 0)
				{
					joy->pushDown();
				}
			}

			if (*(axesstat))
			{
				if (*(axesstat) > 0)
				{
					joy->pushRight();
				}
				else if (*(axesstat) < 0)
				{
					joy->pushLeft();
				}
			}
		}
		else if(type==0)
		{
			joy->releaseA();
			joy->releaseB();
			joy->releaseSelect();
			joy->releaseStart();
			joy->releaseUp();
			joy->releaseDown();
			joy->releaseLeft();
			joy->releaseRight();
		
			bjoy.Update();
			buttonstat = bjoy.ButtonValues();
			bjoy.GetAxisValues(axesstat);
		
			if (buttonstat)
			{
				if ((buttonstat & 2)==2) joy->pushA();
			
				if((buttonstat & 1) == 1) joy->pushB();
			
				if((buttonstat & 4) == 4) joy->pushStart();
			
				if ((buttonstat & 8) == 8) joy->pushSelect();
			}
			if (*(axesstat+1))
			{
				if (*(axesstat+1) > PTOLERANCE)
				{
					joy->pushDown();
				}
				else if (*(axesstat+1) < NTOLERANCE)
				{
					joy->pushUp();
				}
			}

			if (*(axesstat))
			{
				if (*(axesstat) > 0)
				{
					joy->pushRight();
				}
				else if (*(axesstat) < 0)
				{
					joy->pushLeft();
				}
			}
		}

				
	}
	return 0;
}
