/*
** BeNES - Nintendo Entertaiment System Emulator for BeOS
**
** * 1999 by makoto yamagata
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//
// Audio Processo Unit (APU)
//
#ifndef NESAPU_H_DEFINED
#define NESAPU_H_DEFINED

#include "NesTypes.h"
#include "NesSound.h"

class NesAPU {
	NesSound mSound;
	uchar mSq0Regs[4];
	uchar mSq1Regs[4];
	uchar mTriRegs[4];
	uchar mNizRegs[4];
	uchar mVolume[4];
	short mFadeCounter[4];
	short mFadeValue[4];
	short mLength[4];
	short mTriLen;
	short mSweepCounter[2];
	short mSweepValue[2];
	uchar mSweepSpeed[2];
	uchar mSweepDirection[2];
	uchar m4015;
	uchar mToggle;
	bool mSoundEnable;
public:
	NesAPU(): m4015(0), mToggle(0), mSoundEnable(true) { ; }
	void init() { mToggle = 0; }
	uchar read4015() {
		return m4015;
	}
	void enableSound(bool on) { mSoundEnable = on; }
	void reset() {
		mSound.enableChannel(0);
	}
	void write4015(uchar val) {
		mSound.enableChannel(val&0x0f);
	}
	void writeSq0(int regNum, uchar val) { 
		mSq0Regs[regNum] = val;
		switch (regNum) {
		case 0: doSq0Volume(); break;
		case 1: doSq0Sweep(); break;
		case 2: doSq0Freq(); break;
		case 3: doSq0Sound(); break;
		}
	}
	void writeSq1(int regNum, uchar val) { 
		mSq1Regs[regNum] = val;
		switch (regNum) {
		case 0: doSq1Volume(); break;
		case 1: doSq1Sweep(); break;
		case 2: doSq1Freq(); break;
		case 3: doSq1Sound(); break;
		}
	}
	void writeTri(int regNum, uchar val) {
		mTriRegs[regNum] = val;
		switch (regNum) {
		case 0: break;
		case 1: break;
		case 2: doTriFreq(); break;
		case 3: doTriSound(); break;
		}
	}
	void writeNiz(int regNum, uchar val) {
		mNizRegs[regNum] = val;
		switch (regNum) {
		case 0: doNizVolume(); break;
		case 1: break;
		case 2: doNizFreq(); break;
		case 3: doNizSound(); break;
		}
	}
	void doSq0Sound();
	void doSq1Sound();
	void doTriSound();
	void doNizSound();
	void doSq0Volume();
	void doSq1Volume();
	void doNizVolume();
	void doSq0Freq();
	void doSq1Freq();
	void doTriFreq();
	void doNizFreq();
	void doSq0Sweep() { doSqSweep(mSq0Regs, 0); }
	void doSq1Sweep() { doSqSweep(mSq1Regs, 1); }
	void doSqSweep(uchar* regs, int num);
	void statusChange();
};

#endif
